# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
Demonstrate minimal use of the Spark .NET runner functionality.
"""

from shrike.spark import run_spark_net


if __name__ == "__main__":
    run_spark_net()
